@@ -2,9 +2,10 @@
 module containing a parametrized tests testing cross-python
 serialization via the pickle module.
 """
+import textwrap
+
 import py
 import pytest
-import _pytest._code
 
 pythonlist = ["python2.7", "python3.4", "python3.5"]
 
@@ -24,42 +25,44 @@ class Python(object):
     def __init__(self, version, picklefile):
         self.pythonpath = py.path.local.sysfind(version)
         if not self.pythonpath:
-            pytest.skip("%r not found" % (version,))
+            pytest.skip("{!r} not found".format(version))
         self.picklefile = picklefile
 
     def dumps(self, obj):
         dumpfile = self.picklefile.dirpath("dump.py")
         dumpfile.write(
-            _pytest._code.Source(
-                """
-            import pickle
-            f = open(%r, 'wb')
-            s = pickle.dump(%r, f, protocol=2)
-            f.close()
-        """
-                % (str(self.picklefile), obj)
+            textwrap.dedent(
+                """\
+                import pickle
+                f = open({!r}, 'wb')
+                s = pickle.dump({!r}, f, protocol=2)
+                f.close()
+                """.format(
+                    str(self.picklefile), obj
+                )
             )
         )
-        py.process.cmdexec("%s %s" % (self.pythonpath, dumpfile))
+        py.process.cmdexec("{} {}".format(self.pythonpath, dumpfile))
 
     def load_and_is_true(self, expression):
         loadfile = self.picklefile.dirpath("load.py")
         loadfile.write(
-            _pytest._code.Source(
-                """
-            import pickle
-            f = open(%r, 'rb')
-            obj = pickle.load(f)
-            f.close()
-            res = eval(%r)
-            if not res:
-                raise SystemExit(1)
-        """
-                % (str(self.picklefile), expression)
+            textwrap.dedent(
+                """\
+                import pickle
+                f = open({!r}, 'rb')
+                obj = pickle.load(f)
+                f.close()
+                res = eval({!r})
+                if not res:
+                    raise SystemExit(1)
+                """.format(
+                    str(self.picklefile), expression
+                )
             )
         )
         print(loadfile)
-        py.process.cmdexec("%s %s" % (self.pythonpath, loadfile))
+        py.process.cmdexec("{} {}".format(self.pythonpath, loadfile))
 
 
 @pytest.mark.parametrize("obj", [42, {}, {1: 3}])
